/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * os.c
 * This file is part of gedit
 * [Shamelessly copied from time.c]
 * (C) 2005 Muthiah Annamalai.
 *
 * Copyright (C) 2002 Paolo Maggi 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */
 
/*
 * Modified by the gedit Team, 2002. See the AUTHORS file for a 
 * list of people on the gedit Team.  
 * See the ChangeLog files for a list of changes. 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <time.h>

#include <glib/gi18n.h>
#include <glade/glade-xml.h>
#include <gconf/gconf-client.h>
#include <libgnome/gnome-help.h>
#include <libgnome/gnome-config.h>

#include <gedit/gedit-plugin.h>
#include <gedit/gedit-debug.h>
#include <gedit/gedit-menus.h>
#include <gedit/gedit-utils.h>

#include <sys/utsname.h>


#define MENU_ITEM_LABEL		N_("In_sert OS Name")
#define MENU_ITEM_PATH		"/menu/Edit/EditOps_5/"
#define MENU_ITEM_NAME		"PluginInsert OS Name"	
#define MENU_ITEM_TIP		N_("Insert OS Name at the cursor position")

/* For the people to pickup.. */
G_MODULE_EXPORT GeditPluginState update_ui (GeditPlugin *plugin, BonoboWindow *window);
G_MODULE_EXPORT GeditPluginState destroy (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState activate (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState deactivate (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState init (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState configure (GeditPlugin *p, GtkWidget *parent);


static void 
insert_os_name (BonoboUIComponent *uic,gpointer user_data, const gchar* verbname)
{
	struct utsname osname={0,};
	GeditDocument *doc;
	gchar *osdata=NULL;
	
	if(!uname(&osname))
	{
		/* Success */
		osdata=g_strdup_printf(" %s %s %s %s %s ",osname.sysname,
								osname.nodename,
			                      		osname.release,
			                      		osname.version,
			                      		osname.machine);
	}
	else
	{
		osdata=g_strdup_printf(" Unknown OS "); /* Wierdo Error! */
	}

	gedit_debug (DEBUG_PLUGINS, "");
	
	doc = gedit_get_active_document ();
	
	gedit_document_begin_user_action (doc);
	gtk_text_buffer_insert_at_cursor (GTK_TEXT_BUFFER (doc), osdata, -1);
	gedit_document_end_user_action (doc);
	
	g_free(osdata);
}

G_MODULE_EXPORT GeditPluginState
configure (GeditPlugin *p, GtkWidget *parent)
{
	GtkWidget *dlg=NULL;
	gedit_debug (DEBUG_PLUGINS, "Informing Dialog");	
	dlg=gtk_message_dialog_new(NULL,1,2,2,"Hello WOrld!\n This is GEDIT OS Insert plugin",NULL);
	g_return_val_if_fail(dlg!=NULL,PLUGIN_OK);

	gtk_widget_show_all(dlg);
	gtk_dialog_run(dlg);
	gtk_widget_destroy (dlg);

	gedit_debug (DEBUG_PLUGINS, "Done");

	return PLUGIN_OK;
}

G_MODULE_EXPORT GeditPluginState
update_ui (GeditPlugin *plugin, BonoboWindow *window)
{
	BonoboUIComponent *uic;
	GeditDocument *doc;
	GeditMDI *mdi;

	gedit_debug (DEBUG_PLUGINS, "OS Update ui");
	
	g_return_val_if_fail (window != NULL, PLUGIN_ERROR);

	mdi = gedit_get_mdi ();
	g_return_val_if_fail (window != NULL, PLUGIN_ERROR);

	uic = gedit_get_ui_component_from_window (window);

	doc = gedit_get_active_document ();

	if ((doc == NULL) || (gedit_document_is_readonly (doc)) || (gedit_mdi_get_state (mdi) != GEDIT_STATE_NORMAL))		
		gedit_menus_set_verb_sensitive (uic, "/commands/" MENU_ITEM_NAME, FALSE);
	else
		gedit_menus_set_verb_sensitive (uic, "/commands/" MENU_ITEM_NAME, TRUE);

	return PLUGIN_OK;
}
	
G_MODULE_EXPORT GeditPluginState
activate (GeditPlugin *pd)
{
	 /* Standard stuff */
	 GList *top_windows;
        gedit_debug (DEBUG_PLUGINS, "");

        top_windows = gedit_get_top_windows ();
        g_return_val_if_fail (top_windows != NULL, PLUGIN_ERROR);

        while (top_windows)
        {
		gedit_menus_add_menu_item (BONOBO_WINDOW (top_windows->data),
				     MENU_ITEM_PATH, MENU_ITEM_NAME,
				     MENU_ITEM_LABEL, MENU_ITEM_TIP,
				     NULL, insert_os_name);

                pd->update_ui (pd, BONOBO_WINDOW (top_windows->data));

                top_windows = g_list_next (top_windows);
        }
       
	
	return PLUGIN_OK;
}

G_MODULE_EXPORT GeditPluginState
deactivate (GeditPlugin *pd)
{
	gedit_debug (DEBUG_PLUGINS, "Deactivated OS plugin");

	gedit_menus_remove_menu_item_all (MENU_ITEM_PATH, MENU_ITEM_NAME);

	return PLUGIN_OK;
}

G_MODULE_EXPORT GeditPluginState
destroy (GeditPlugin *pd)
{
	gedit_debug (DEBUG_PLUGINS, "Closing OS Plugin");
	return PLUGIN_OK;
}

G_MODULE_EXPORT GeditPluginState
init (GeditPlugin *pd)
{
	/* initialize */
	gedit_debug (DEBUG_PLUGINS, "OS Init plugin");

	pd->private_data = NULL;
		
	/* NO GConf Stuff */	
	return PLUGIN_OK;
}

